package logic

import (
	"fusenapi/constants"
	"fusenapi/utils/auth"
	"fusenapi/utils/basic"
	"fusenapi/utils/file"
	"fusenapi/utils/websocket_data"
	"time"

	"context"

	"fusenapi/server/websocket/internal/svc"
	"fusenapi/server/websocket/internal/types"

	"github.com/zeromicro/go-zero/core/logx"
)

type RenderNotifyLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewRenderNotifyLogic(ctx context.Context, svcCtx *svc.ServiceContext) *RenderNotifyLogic {
	return &RenderNotifyLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

// 处理进入前逻辑w,r
// func (l *RenderNotifyLogic) BeforeLogic(w http.ResponseWriter, r *http.Request) {
// }

// 处理逻辑后 w,r 如:重定向, resp 必须重新处理
// func (l *RenderNotifyLogic) AfterLogic(w http.ResponseWriter, r *http.Request, resp *basic.Response) {
// // httpx.OkJsonCtx(r.Context(), w, resp)
// }

func (l *RenderNotifyLogic) RenderNotify(req *types.RenderNotifyReq, userinfo *auth.UserInfo) (resp *basic.Response) {
	if req.TaskId == "" {
		logx.Error("渲染回调参数错误：invalid param task_id")
		return resp.SetStatusWithMessage(basic.CodeRequestParamsErr, "invalid param task_id")
	}
	if req.Image == "" {
		logx.Error("渲染回调参数错误：invalid param image")
		return resp.SetStatusWithMessage(basic.CodeRequestParamsErr, "invalid param image")
	}
	// 上传文件
	var upload = file.Upload{
		Ctx:        l.ctx,
		MysqlConn:  l.svcCtx.MysqlConn,
		AwsSession: l.svcCtx.AwsSession,
	}
	uploadRes, err := upload.UploadFileByBase64(&file.UploadBaseReq{
		Source:       "unity cloud render",
		FileHash:     req.TaskId,
		FileData:     req.Image,
		Metadata:     "",
		UploadBucket: 1,
		ApiType:      2,
		UserId:       req.UserId,
		GuestId:      req.GuestId,
		FileByte:     nil,
	})
	if err != nil {
		logx.Error("渲染回调上传文件失败：", err)
		return resp.SetStatusWithMessage(basic.CodeFileUploadErr, "failed to upload render resource image")
	}
	//遍历websocket链接把数据传进去
	mapConnPool.Range(func(key, value any) bool {
		//断言连接
		ws, ok := value.(wsConnectItem)
		if !ok {
			logx.Error("渲染回调断言websocket连接失败")
			return true
		}
		//关闭标识
		if ws.isClose {
			return true
		}
		//查询有无该渲染任务
		renderId, ok := ws.renderProperty.renderImageTask[req.TaskId]
		if !ok {
			return true
		}
		b := ws.respondDataFormat(constants.WEBSOCKET_RENDER_IMAGE, websocket_data.RenderImageRspMsg{
			RenderId: renderId,
			Image:    uploadRes.ResourceUrl,
		})
		deleteTask := renderImageControlChanItem{
			Option:   0, //0删除 1添加
			TaskId:   req.TaskId,
			RenderId: renderId,
		}
		select {
		case <-ws.closeChan: //关闭了
			return true
		case ws.renderProperty.renderImageTaskCtlChan <- deleteTask: //删除对应的需要渲染的图片map
			//发送数据到out chan
			ws.sendToOutChan(b)
			return true
		case <-time.After(time.Second * 3): //超时丢弃
			return true
		}
	})
	logx.Info("渲染回调成功######################")
	return resp.SetStatusWithMessage(basic.CodeOK, "success")
}
